#include <unistd.h>
#include <sys/socket.h>
#include <sys/errno.h>
#include "netport.h"

/* these guys come in handy on both server and client */

/* return -1 on failure, 0 if we read length bytes successfully */
/* errno and such should still be valid if we return with an error */
int read_or_fail(int fd, unsigned char buf[], int length)
{
    int readsofar;
    int result;

    readsofar = 0;
    while (readsofar < length) {
        while ((result = recv(fd, buf+readsofar, length-readsofar, 0)) < 0) {
            if (errno != EINTR) {
                return -1;
            }
        }
	/* if the connection was closed */
	if (result == 0) return -1;
	/* otherwise note what we received */
        if (result > 0) readsofar += result;
    }

    return 0;
}

int write_or_fail(int fd, const unsigned char buf[], int length)
{
    int wrotesofar;
    int result;

    wrotesofar = 0;
    while (wrotesofar < length) {
        while ((result = send(fd, buf+wrotesofar, length-wrotesofar, 0)) < 0) {
            if (errno != EINTR) {
                return -1;
            }
        }
	/* detect disconnect */
	if (result == 0) return -1;
	/* otherwise note what we got */
        if (result > 0) wrotesofar += result;
    }

    return 0;
}

